/*
This file contains the list of the upper-case Greek letters.
In case I overlooked any, it can be added here.
*/
char *GRK_list[] =
{
"ALPHA",    "BETA",    "CHI",    "DELTA",   "EPSILON",
"ETA",      "GAMMA",   "KAPPA",  "LAMDA",   "MU",
"NU",       "OMEGA",   "PHI",    "PI",      "PSI",
"RHO",      "SIGMA",   "TAU",    "THETA",   "XI",    "ZETA"
};
