c      
c     SUBROUTINE h
c      
      subroutine h(q,qd,u,param,alfa,beta,gama,fmat)
        doubleprecision q(23),qd(23),u(2),param(20),alfa,beta,gama
        implicit doubleprecision (t)
        doubleprecision fmat(43,1)
