c      
c     SUBROUTINE c
c      
      subroutine c(q,qd,u,paramopt,fmat)
        doubleprecision q(23),qd(23),u(2),paramopt(4)
        implicit doubleprecision (t)
        doubleprecision fmat(1,1)
         fmat(1,1) = 0
      end
